use core::fmt;
use std::error::Error;

#[derive(Debug)]
pub struct MessageError(Option<String>);

// 实现Error trait
impl Error for MessageError {

    fn cause(&self) -> Option<&dyn Error> {
        self.source()
    }

}

// 实现Display trait，这是Error trait的一部分要求
impl fmt::Display for MessageError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match &self.0 {
            Some(message) => write!(f, "{}", message),
            None => write!(f, "")
        }
    }
}

impl From<String> for MessageError {
    fn from(value: String) -> Self {
        MessageError(Some(value))
    }
}

impl From<&str> for MessageError {
    fn from(value: &str) -> Self {
        MessageError(Some(value.to_string()))
    }
}